#ifndef TA_TYPES_WRAPPER_H
#define TA_TYPES_WRAPPER_H

#define TA_MSVC



#ifdef LIBRARIESDLL_EXPORTS
#define TA_EXPORT_DLL
#endif

#ifdef TA_EXPORT_DLL
#define TA_EXPORT __declspec(dllexport)
#define TA_CALL __stdcall
#else
#define TA_EXPORT __declspec(dllimport)
#define TA_CALL __stdcall
#endif

#ifdef TA_MSVC
#define TA_ALIGN_16 __declspec(align(16))
#else
#define TA_ALIGN_16
#endif

#if defined(__cplusplus) && !defined(TA_C_IMPLEMENTATION)
namespace TA
{

class Physics;
class DynamicObject;
class StaticObject;
class PhysicsJoint;
class Collision;
class CollisionObjectCombo;
class CollisionObjectComplex { void* m_pInst; };
class CollisionObject { void* m_pInst; };

struct LineIntersection;
struct SurfaceAttribute;

struct MFrame;
struct Mat33;
struct Vec3;
struct EulerAngles;
struct AABB;

typedef bool (TA_CALL *DynamicVsDynamicCollisionTestCallBack)(DynamicObject& objectA, DynamicObject& objectB);
typedef bool (TA_CALL *DynamicVsStaticCollisionTestCallBack)(DynamicObject& objectA, StaticObject& objectB);
typedef bool (TA_CALL *PreProcessCollisionCallBack)(Collision& collision);
typedef void (TA_CALL *PostProcessCollisionCallBack)(const Collision& collision);

#ifdef TA_MSVC

typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;
typedef unsigned __int64 u64;

typedef signed char s8;
typedef short s16;
typedef int s32;
typedef __int64 s64;

typedef char Char;

#endif
#ifdef GCC

typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;
typedef unsigned long long  u64;

typedef signed char s8;
typedef short s16;
typedef int s32;
typedef long long s64;

typedef char Char;

#endif

} // namespace TA


#define TACOMMON_CLASS
//#define TAC_CALL __fastcall
//#define TA_FAST_CALL __fastcall
#include "MFrame.h"
#include "AABB.h"

#endif // defined(__cplusplus) && !defined(TA_C_IMPLEMENTATION)

#if defined(TA_C_IMPLEMENTATION)
#define DynamicVsDynamicCollisionTestCallBack Physics::DynamicVsDynamicCollisionTestCallBack
#define DynamicVsStaticCollisionTestCallBack Physics::DynamicVsStaticCollisionTestCallBack
#define PreProcessCollisionCallBack Physics::PreProcessCollisionCallBack
#define PostProcessCollisionCallBack Physics::PostProcessCollisionCallBack
#endif // defined(TA_C_IMPLEMENTATION)

#ifdef TA_MSVC

typedef unsigned char ta_u8;
typedef unsigned short ta_u16;
typedef unsigned int ta_u32;
typedef unsigned __int64 ta_u64;

typedef signed char ta_s8;
typedef short ta_s16;
typedef int ta_s32;
typedef __int64 ta_s64;

#endif

#ifdef TA_GCC


typedef unsigned char ta_u8;
typedef unsigned short ta_u16;
typedef unsigned int ta_u32;
typedef unsigned long long  ta_u64;

typedef signed char ta_s8;
typedef short ta_s16;
typedef int ta_s32;
typedef long long ta_s64;


#endif

typedef struct _TaDynamicObject { void* pEmpty; } TaDynamicObject;
typedef struct _TaCollisionObjectCombo { void* pEmpty; } TaCollisionObjectCombo;
typedef struct _TaLineIntersection { void* pEmpty; } TaLineIntersection;
typedef struct _TaPhysics { void* pEmpty; } TaPhysics;
typedef struct _TaStaticObject { void* pEmpty; } TaStaticObject;
typedef struct _TaPhysicsJoint { void* pEmpty; } TaPhysicsJoint;
typedef struct _TaCollision { ta_u8 data[128]; } TaCollision;
typedef struct _TaSurfaceAttribute { void* pEmpty; } TaSurfaceAttribute;
typedef struct _TaCollisionObjectComplex { void* pEmpty; } TaCollisionObjectComplex;
typedef struct _TaCollisionObjectAABBMesh { void* pEmpty; } TaCollisionObjectAABBMesh;
typedef struct _TaCollisionObject{ void* pEmpty; } TaCollisionObject;

typedef char TaChar;
typedef unsigned char TaBool;
const int TaTrue = 1;
const int TaFalse = 0;

typedef TA_ALIGN_16 struct _TaVec3
{
	float x;
	float y;
	float z;
	float padding;
} TaVec3;
typedef struct _TaAABB
{
	TaVec3 v3Center;
	TaVec3 v3Extent;
} TaAABB;
typedef struct _TaMat33
{
	TaVec3 v3X;
	TaVec3 v3Y;
	TaVec3 v3Z;
} TaMat33;
typedef struct _TaMFrame
{
	TaMat33 m33Rotation;
	TaVec3 v3Translation;
} TaMFrame;
typedef struct _TaEulerAngles
{
	float x;
	float y;
	float z;
} TaEulerAngles;


typedef TaBool (TA_CALL *TaDynamicVsDynamicCollisionTestCallBack)(TaDynamicObject* objectA, TaDynamicObject* pObjectB);
typedef TaBool (TA_CALL *TaDynamicVsStaticCollisionTestCallBack)(TaDynamicObject* objectA, TaStaticObject* pObjectB);
typedef TaBool (TA_CALL *TaPreProcessCollisionCallBack)(TaCollision* pCollision);
typedef void (TA_CALL *TaPostProcessCollisionCallBack)(const TaCollision* pCollision);



#endif // TA_TYPES_WRAPPER_H
